print("import: 'altair'")
import altair

print("import: 'altair.examples'")
import altair.examples

print("import: 'altair.expr'")
import altair.expr

print("import: 'altair.sphinxext'")
import altair.sphinxext

print("import: 'altair.tests'")
import altair.tests

print("import: 'altair.utils'")
import altair.utils

print("import: 'altair.vega'")
import altair.vega

print("import: 'altair.vega.v5'")
import altair.vega.v5

print("import: 'altair.vega.v5.schema'")
import altair.vega.v5.schema

print("import: 'altair.vegalite'")
import altair.vegalite

print("import: 'altair.vegalite.v3'")
import altair.vegalite.v3

print("import: 'altair.vegalite.v3.schema'")
import altair.vegalite.v3.schema

print("import: 'altair.vegalite.v4'")
import altair.vegalite.v4

print("import: 'altair.vegalite.v4.schema'")
import altair.vegalite.v4.schema

